// FILE: IMUSensor.h
#if defined(ARDUINO) && (ARDUINO >= 100)
	#include <Arduino.h>
#else
	#include <WProgram.h>
#endif

#include "I2Cdev.h"

#include "MPU6050_6Axis_MotionApps20.h"

#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
    #include "Wire.h"
#endif

MPU6050 mpu;

bool dmpReady = false;  // set true if DMP init was successful
uint8_t devStatus;      // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;    // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;     // count of all bytes currently in FIFO
uint8_t fifoBuffer[64]; // FIFO storage buffer

// orientation/motion vars
Quaternion q;           // [w, x, y, z]         quaternion container
VectorFloat gravity;    // [x, y, z]            gravity vector
float ypr[3];           // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector
int rollVal, pitchVal, yawVal;
int sumRoll = 0, sumPitch=0, sumYaw=0;

void mpuBegin() {
	
    // join I2C bus (I2Cdev library doesn't do this automatically)
    #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
        Wire.begin();
        Wire.setClock(400000); // 400kHz I2C clock. Comment this line if having compilation difficulties
    #elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
        Fastwire::setup(400, true);
    #endif
	
    mpu.initialize();

    devStatus = mpu.dmpInitialize();

    // set gyro offsets h
    mpu.setXGyroOffset(220);
    mpu.setYGyroOffset(76);
    mpu.setZGyroOffset(-85);
    mpu.setZAccelOffset(1788); // 1688 factory default for my test chip

    // make sure it worked (returns 0 if so)
    if (devStatus == 0) {
        mpu.setDMPEnabled(true);
        dmpReady = true;

        // get expected DMP packet size for later comparison
        packetSize = mpu.dmpGetFIFOPacketSize();
    } 

}

void getRollPitchYaw() {
    // if programming failed, don't try to do anything
    if (!dmpReady) return;

    // get current FIFO count
    fifoCount = mpu.getFIFOCount();

     // wait for correct available data length, should be a VERY short wait
    while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();

    // read a packet from FIFO
    mpu.getFIFOBytes(fifoBuffer, packetSize);
    
    // track FIFO count here in case there is > 1 packet available
    fifoCount -= packetSize;
    
    mpu.dmpGetQuaternion(&q, fifoBuffer);
    mpu.dmpGetGravity(&gravity, &q);
    mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);
	
	yawVal = ypr[0] * 180/M_PI;
	pitchVal = ypr[1] * 180/M_PI;
	rollVal = ypr[2] * 180/M_PI;
}

int readRPYData(int axis){
	for(int i=0;i<10;i++)
	{
		getRollPitchYaw();
		sumRoll = sumRoll + rollVal;
		sumPitch = sumPitch + pitchVal;
		sumYaw = sumYaw + yawVal;
	}
	rollVal = sumRoll/10;
	pitchVal = sumPitch/10;
	yawVal = sumYaw/10;
	
	sumRoll = 0;
	sumPitch = 0;
	sumYaw = 0;
	
	switch(axis)
	{
		case 1: 
			return rollVal;
			break;
		case 2: 
			return pitchVal;
			break;
		case 3: 
			return yawVal;
			break;
		default:
			break;
	}
}
// END OF FILE